@@ -19,17 +19,17 @@
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # Standard Python modules
-import sys, os, optparse, pwd
-import time, select, popen2
+import sys, os, optparse, pwd, grp
+import time, select, popen2, re
 import audioop, signal, inspect
-import tempfile, shutil, syslog
+import tempfile, shutil, syslog, errno
 
 # External phonepatch modules
 sys.path.append("/usr/lib/asterisk-phonepatch")
 import templateparser, dtmf, radio
 import radiocontrol, daemonize
 
-__version__ = "$Revision: 1.3 $"
+__version__ = "$Revision: 1.4 $"
 __author__ = "Arnau Sanchez <arnau@ehas.org>"
 __depends__ = ['Asterisk', 'Sox', 'Festival', 'Python-2.4']
 __copyright__ = """Copyright (C) 2006 Arnau Sanchez <arnau@ehas.org>.
@@ -123,7 +123,7 @@ class Phonepatch:
 		SIGTERM/SIGKILL: Kill phonepatch daemon()
 		"""
 		signame = self.signals.get(signum, "unknown")
-		self.debug("signal_handler - received %s" %signame)
+		self.debug("signal_handler: received %s" %signame)
 		
 		if signum == signal.SIGUSR1 and not self.sleep_signal:
 			self.debug("signal_handler: phonepatch daemon paused")
@@ -225,23 +225,26 @@ class Phonepatch:
 		soundcard = self.configuration["soundcard"]
 		
 		control = radioconf["radio_control"]
+		self.ptt, self.carrier_detection = radioconf["ptt"], radioconf["carrier_detection"]
 		self.radio_control = None
-		if control != "off":
-			param = control.split(":")
-			if control.find("serial:") == 0:
-				self.radio_control = radiocontrol.RadioControl("serial", param[1])
-			elif control.find("parallel:") == 0:
-				self.radio_control = radiocontrol.RadioControl("parallel",  param[1])
-			elif control.find("command:") == 0:
-				self.command_options = {"set_ptt_on": radioconf["ptt_on"], \
-					"set_ptt_off": radioconf["ptt_off"],
-					"get_carrier": radioconf["get_carrier"], 
-					"get_carrier_response": radioconf["get_carrier_response"], }
-				self.radio_control = radiocontrol.RadioControl("command",  param[1], self.command_options)
+		if control != "off" and (self.ptt or self.carrier_detection):
+			try: dtype, lines, device = re.findall("(serial|parallel|command)(.*):(.*)$", control)[0]
+			except: self.debug("syntax error on radio_control: %s" %control, exit = 1)
+			if lines and lines[0] == "[" and lines[-1] == "]": lines = [x.strip() for x in lines[1:-1].split(",")]
+			else: lines = None
+			if control.find("serial") == 0:
+				self.radio_control = radiocontrol.RadioControl("serial", device, lines)
+			elif control.find("parallel") == 0:
+				self.radio_control = radiocontrol.RadioControl("parallel", device, lines)
+			elif control.find("command") == 0:
+				self.command_options = {"set_ptt_on": radioconf["command_ptt_on"], \
+					"set_ptt_off": radioconf["command_ptt_off"],
+					"get_carrier": radioconf["command_get_carrier"], 
+					"get_carrier_response": radioconf["command_get_carrier_response"], }
+				self.radio_control = radiocontrol.RadioControl("command", device, command_options=self.command_options)
 			else:
 				self.debug("syntax error on radio_control: %s" %control, exit = 1)
 	
-		self.ptt, self.carrier_detection = radioconf["ptt"], radioconf["carrier_detection"]
 		# Create radio instance (control soundcard and PTT)
 		try:self.radio = radio.Radio(soundcard["device"], self.samplerate, self.asterisk_samplerate, \
 			self.radio_control, self.ptt, self.carrier_detection, verbose=self.modules_verbose, \
@@ -255,7 +258,7 @@ class Phonepatch:
 		except IOError, detail:
 			self.debug("%s" %str(detail), 1)
 			sys.exit(1)
-		
+			
 		self.debug("soundcard opened: %s (%s sps)" %(soundcard["device"], self.samplerate))
 		if self.radio_control:
 			self.debug("radio control opened: %s" %control)
@@ -526,7 +529,8 @@ class Phonepatch:
 		if not self.pidfile: return
 		self.debug("deleting pidfile: %s" %self.pidfile)
 		try: os.unlink(self.pidfile)
-		except: pass
+		except OSError, e: 
+			if e.errno != errno.ENOENT: raise
 
 	###################################
 	def create_pidfile(self):
@@ -536,6 +540,7 @@ class Phonepatch:
 		except: self.debug("pidfile could not be opened for writing", exit = 1, delete_pidfile = False)
 		fd.write(str(os.getpid()) + "\n")
 		fd.close()
+		os.chown(self.pidfile, *self.get_asterisk_id())
 
 	###################################
 	def read_pidfile(self):
@@ -600,17 +605,17 @@ class Phonepatch:
 			os.write(tempfd, buffer + "\n")
 			self.debug("outcall - %s" %buffer)
 		os.close(tempfd)
-		uid, gid = pwd.getpwnam("asterisk")[2:4]
+		
+		# Spool file must be owned by Asterisk
+		uid, gid = self.get_asterisk_id()
+		os.chown(callpath, uid, gid)
 		
 		# Now make the outcall and wait for asterisk response
 		self.debug("making an outcall")
-		sfile = os.path.join(self.outcalls_dir, os.path.basename(callpath))
-		shutil.move(callpath, self.outcalls_dir)
+		callspool = os.path.join(self.outcalls_dir, os.path.basename(callpath))
+		os.rename(callpath, callspool)
 		
-		# Must be owned by Asterisk
-		os.chown(sfile, uid, gid)
-	
-		# Save outcall spool file name on class, as callback continue_outcall() uses it
+		# Save outcall spool file name on class object, as callback continue_outcall() uses it
 		self.outcallfile = os.path.join(self.outcalls_dir, os.path.basename(callpath))
 		
 		# Loop until spool file is processed or a <sleep_signal> received
@@ -686,6 +691,10 @@ class Phonepatch:
 			return
 		return pid
 
+	###################################
+	def get_asterisk_id(self):
+		return pwd.getpwnam("asterisk")[2:4]
+
 	###################################
 	def init_daemon(self):
 		if self.background: 
@@ -703,7 +712,14 @@ class Phonepatch:
 		self.sleep_signal = self.stopped = False
 		self.set_signals([signal.SIGHUP, signal.SIGUSR1, signal.SIGUSR2, \
 			signal.SIGALRM, signal.SIGTERM, signal.SIGINT])
-	
+		self.create_pidfile()
+
+		uid, gid = self.get_asterisk_id()
+		asterisk_groups = [x[2] for x in grp.getgrall() if "asterisk" in x[3]]
+		os.setgroups([gid] + asterisk_groups)
+		os.setregid(uid, uid)
+		os.setreuid(gid, gid)
+		
 	###################################
 	def process_noisy_number(self, number, noisy_button):
 		"""All repetitions between a noisy_button are 
@@ -801,7 +817,6 @@ class Phonepatch:
 		self.set_state("daemon")
 		self.init_daemon()
 		self.open_radio()
-		self.create_pidfile()
 		
 		if testcall != None:
 			self.make_call(self.configuration["outcall"], testcall, reopen = False)
@@ -878,7 +893,7 @@ and spawn outgoing calls when required (radio DTMF controlled)"""
 
 	config.read_template(default_template)
 	try: configuration = config.read_configuration(options.configuration_file)
-	except IOError, e: print "Error reading configuration file: %s" %options.configuration_file; sys.exit(1)
+	except IOError, e: sys.stderr.write("Error reading configuration file: %s\n" %options.configuration_file); sys.exit(1)
 
 	# Run daemon (default), incall or outcall mode
 	php = Phonepatch(configuration, verbose=options.verbose)
